#include <ncbi_pch.hpp>
#include "cseqcoding.hpp"

USING_OLIGOFAR_SCOPES;

char CIupacnaBase::s_complement[] = 
    "................""................"
    ".............-..""................"
    ".TVGH..CD..M.KN.""..YSAABW.R......"
    ".tvgh..cd..m.kn.""..ysaabw.r......"
    "................""................"
    "................""................"
    "................""................"
    "................""................";

char CNcbi8naBase::s_complement[] = "\x0\x8\x4\xc\x2\xa\x6\xe\x1\x9\x5\xd\x3\xb\x7\xf";	

char CNcbi8naBase::s_altCount[] = "\x0\x1\x1\x2""\x1\x2\x2\x3""\x1\x2\x2\x3""\x2\x3\x3\x4";

char CNcbi8naBase::s_fromIupacna[] =     
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x00-0x0f
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x10-0x1f
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\x00\xf0\xf0" // 0x20-0x2f
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x30-0x3f
    "\xf0\x01\x0e\x02""\x0d\xf0\xf0\x04""\x0b\xf0\xf0\x0c""\xf0\x03\x0f\xf0" // 0x40-0x4f
    "\xf0\xf0\x05\x06""\x08\x08\x07\x09""\xf0\x0a\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x50-0x5f
    "\xf0\x01\x0e\x02""\x0d\xf0\xf0\x04""\x0b\xf0\xf0\x0c""\xf0\x03\x0f\xf0" // 0x60-0x6f
    "\xf0\xf0\x05\x06""\x08\x08\x07\x09""\xf0\x0a\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x70-0x7f
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x80-0x8f
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x90-0x9f
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0xa0-0xaf
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0xb0-0xbf
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0xc0-0xcf
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0xd0-0xdf
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0xe0-0xef
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0xf0-0xff
    ;

char CNcbi8naBase::s_smallestNcbi2na[] = "\x0\x0\x1\x0""\x2\x0\x1\x0""\x3\x0\x1\x0""\x2\x0\x1\x0";

char CNcbi2naBase::s_iupacnaTable[] = 
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x00-0x0f
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x10-0x1f
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x20-0x2f
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x30-0x3f
    "\xf0\x00\x01\x01""\xf0\xf0\xf0\x02""\xf0\xf0\xf0\x02""\xf0\xf0\xf0\xf0" // 0x40-0x4f
    "\xf0\xf0\xf0\x01""\x03\x03\xf0\xf0""\xf0\x01\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x50-0x5f
    "\xf0\x00\x01\x01""\xf0\xf0\xf0\x02""\xf0\xf0\xf0\x02""\xf0\xf0\xf0\xf0" // 0x60-0x6f
    "\xf0\xf0\xf0\x01""\x03\x03\xf0\xf0""\xf0\x01\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x70-0x7f
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x80-0x8f
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0x90-0x9f
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0xa0-0xaf
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0xb0-0xbf
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0xc0-0xcf
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0xd0-0xdf
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0xe0-0xef
    "\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0""\xf0\xf0\xf0\xf0" // 0xf0-0xff
    ;

char CNcbi2naBase::s_ncbi8naTable[] = 
    "\xf0\x00\x01\xf0""\x02\xf0\xf0\xf0""\x03\xf0\xf0\xf0""\xf0\xf0\xf0\xf0";

char CColorTwoBase::s_dibase2code[] = "\x00\x10\x20\x30""\x10\x00\x30\x20""\x20\x30\x00\x10""\x30\x20\x10\x00";

